# Generated by Django 2.2.3 on 2019-09-09 15:07

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='currentsong',
            name='manually_requested',
            field=models.BooleanField(default=True),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='playlog',
            name='manually_requested',
            field=models.BooleanField(default=True),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='queuedsong',
            name='manually_requested',
            field=models.BooleanField(default=True),
            preserve_default=False,
        ),
    ]
