# Generated by Django 2.2.3 on 2019-09-13 15:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0004_archivedplaylist_created'),
    ]

    operations = [
        migrations.AlterField(
            model_name='archivedplaylist',
            name='list_id',
            field=models.CharField(max_length=200),
        ),
    ]
